#pragma once

#include "docopt_args.h"

#include <stdbool.h>
#include <stdint.h>


char const *{{rendering.module_prefix}}_processCommand(uint8_t argc, char **argv);
char const *{{rendering.module_prefix}}_getPrompt(void);
char const *{{rendering.module_prefix}}_getHelpText(void);

// Command Handlers

char const *{{rendering.module_prefix}}_handle_Help(DocoptArgs *args);
char const *{{rendering.module_prefix}}_handle_Error(DocoptArgs *args);

// AUTOGENERATED START
{% for command in rendering.commands -%}
// > {{command.docopt_text}}
char const *{{rendering.module_prefix}}_handle_{{command.function_name}}(DocoptArgs *args);
{% endfor -%}
// AUTOGENERATED End

